import datetime, os, urllib

from sqlalchemy import Boolean, Column, Date, DateTime, Float, Integer, String, Text, Time, ForeignKey
from libraries.database.database import db


# ------------------------------------------------------------------------------------------ #
class BroadcastModel:
    def store(self, commit=False):
        db.session.add(self)
        if commit:
            self.commit()

    def commit(self):
        db.session.commit()

# ------------------------------------------------------------------------------------------ #
class Format(db.Model, BroadcastModel):
    """
    ID of show format # Sendungsformat
    """
    __tablename__ = 'format'

    id = Column(Integer, primary_key=True)
    name = Column(String(255))

# ------------------------------------------------------------------------------------------ #
class BroadcastPeriod(db.Model, BroadcastModel):
    """
    Daten werden noch in der scheduling.xml gehalten
    """
    __tablename__  = 'broadcast_period'

    identifier = Column(Integer, primary_key=True)
    start = Column(DateTime)
    end = Column(DateTime)

    meta = {
        'ordering': ['+start']
    }


# ------------------------------------------------------------------------------------------ #
class BroadcastEvent(db.Model, BroadcastModel):
    """
    Definiert eine Sendung mit Start- und End-Datum
    """
    __tablename__ = 'broadcast_event'

    identifier = Column(Integer, primary_key=True, autoincrement=True) # Unique (o)id
    id_ext = Column(String(30)) #
    location = Column(String(512)) # archived audio file, will be available when the broadcast event is over
    reccurrence_id = Column(Integer) # Unique id of event to be repeated
    duration = Column(Integer) # duration in seconds
    start = Column(DateTime, nullable=False, default=datetime.datetime.now())  # start date
    end = Column(DateTime, nullable=False, default=datetime.datetime.now()+datetime.timedelta(minutes=60))  # end date
    rerun = Column(Boolean, default=False)  # true, if the event is a rerun
    replay_of_datetime = Column(DateTime)
    replay_of = Column(Integer, ForeignKey('broadcast_event.identifier'))
    programme_id = Column(String(100))
    station_name = Column(String(100))
    station_id = Column(String(100))
    title = Column(String(100))
    subject = Column(String(200))
    description = Column(String(300))
    overwrite_event = Column(Integer, ForeignKey('broadcast_event.identifier'))
    state = Column(String(50), default='created')
    created = Column(DateTime, default=datetime.datetime.now)
    modified = Column(DateTime, default=datetime.datetime.now)
    modified_by = Column(String(50))
    text = Column(String(400))
    #data = DictField(required=False)

    meta = {
        'ordering': ['+start'],
        'indexes': [
            {'fields': ['title', "subject"],
             'default_language': 'german',
             'weight': {'title': 10, 'subject': 2}
             }
        ]
    }

    def fileExists(self):
        return os.path.exists(str(self.location).replace('file://', ''))


# ------------------------------------------------------------------------------------------ #
class BroadcastEventTrack(db.Model, BroadcastModel):
    """
    Track, der einem BroadcastEvent zugeordnet ist
    """
    __tablename__ = 'broadcast_event_track'

    identifier = Column(Integer, primary_key=True) # Unique id
    location = Column(String(512))  # audio location (url or file)
    length = Column(Float) # duration in seconds
    start = Column(DateTime) # start date
    end = Column(DateTime) # end date
    record_at = Column(DateTime) # The date on which the recording has to start.
    broadcast_event = Column(Integer, ForeignKey('broadcast_event.identifier')) # the BroadcastEvent the track refers to

    meta = {
        'ordering': ['+start']
    }

    def isLast(self):
        return self == BroadcastEventTrack.objects(broadcast_event=self.broadcast_event).order_by('-start').first()

    def fileExists(self):
        return os.path.exists(str(self.location).replace('file://', ''))

    def totalLength(self):
        tracks = BroadcastEventTrack.objects(broadcast_event=self.broadcast_event)
        length = 0
        for track in tracks:
            length = length + track.length if track.length else length
        return length#


# ------------------------------------------------------------------------------------------ #
class BroadcastEventOverride(db.Model, BroadcastModel): # Base
    """
    Ein Track, der in die Sendeautomatisierung eingeblendet wird
    Muss manuell erstellt werden
    """
    __tablename__ = 'broadcast_event_override'

    identifier = Column(Integer, primary_key=True) # Unique id
    location = Column(String(512)) # audio location (url or file)
    mimetype = Column(String(100)) # audio mime
    bitrate = Column(Integer) # bitrate
    channels = Column(Integer) # number of channels
    length = Column(Float) # duration in seconds
    start = Column(DateTime) # start date
    end = Column(DateTime) # end date
    seek = Column(Integer) # TODO: seconds to seek into the audio
    ordering = Column(Integer, default=0)  # track sorting
    #data = DictField(required=False) # additional data
    broadcast_event = Column(Integer, ForeignKey('broadcast_event.identifier'))  # the BroadcastEvent the track refers to

    meta = {
        'ordering': ['+ordering']
    }

    def nextOrdering(self):
        """
        Return number of next track in tracklist
        @return:  int ordering
        """
        lastOverride = BroadcastEventOverride.objects(broadcast_event=self.broadcast_event)\
            .order_by('-ordering')\
            .first()

        if lastOverride:
            return lastOverride.ordering + 1
        else:
            return 1

    def filled(self):
        """
        what percent of the broadcast event is allready filled with audio
        @return:  percent filled with audio
        """
        object = self.broadcast_event
        parent_total = (object.end - object.start).total_seconds()
        proc = 0
        try:
            proc = (self.totalLength() / parent_total) * 100
        except:
            proc = 0
        return int(proc)

    def totalLength(self):
        """
        Get the total length of audio in the track list
        @return:  total length of audio
        """
        tracks = BroadcastEventOverride.objects(broadcast_event=self.broadcast_event)
        length = 0
        for track in tracks:
            if track.length:
                length = length + track.length
        return length

    def fileExists(self):
        request = urllib.Request(self.location)
        request.get_method = lambda: 'HEAD'
        try:
            response = urllib.urlopen(request)
            return True
        except:
            return False
